/** @file HL_sys_main.c 
*   @brief Application main file
*   @date 11-Dec-2018
*   @version 04.07.01
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2018 Texas Instruments Incorporated - www.ti.com  
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
#include "HL_crc.h"
#include <stdio.h>
/* USER CODE END */

/* Include Files */

#include "HL_sys_common.h"

/* USER CODE BEGIN (1) */
void Data_Initialization(void);
uint64_t swap_high32_with_low32( uint64_t val);
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
#define SECTOR_COUNT    1
#define DATA_SIZE 128
uint64_t Data_Buffer[DATA_SIZE];
uint64_t Pre_Determined_CRC[SECTOR_COUNT];
uint64_t compressed_crc;

crcConfig_t sCrcParams;

crcModConfig_t deneme;

uint8_t CRC_Completion_Flag = 0;
/* USER CODE END */

int main(void)
{
/* USER CODE BEGIN (3) */
    crcInit();
    Data_Initialization();

    printf("Start CRC Calculation in Full-CPU Mode (IRQ based)\n");

    sCrcParams.crc_channel   = CRC_CH1;
    sCrcParams.mode          = CRC_FULL_CPU;
    sCrcParams.pcount        = 0u;
    sCrcParams.scount        = 0u;
    sCrcParams.wdg_preload   = 0u;
    sCrcParams.block_preload = 0u;

    crcChannelReset(crcREG1, CRC_CH1);
    crcSetConfig(crcREG1, &sCrcParams);

    deneme.mode             = CRC_FULL_CPU;
    deneme.crc_channel      = CRC_CH1;
    deneme.src_data_pat     = Data_Buffer;
    deneme.data_length      = DATA_SIZE;

    crcSignGen(crcREG1, &deneme);
    compressed_crc  =   crcGetPSASig(crcREG1, CRC_CH1);/*Reading the compressed CRC from the CRC module*/
    compressed_crc  =   swap_high32_with_low32(compressed_crc); /*Due to bug in crcGetPSASig the received CRC need to swap again*/
    if(compressed_crc == Pre_Determined_CRC[0])
    {
        printf("CRC Verification Successful\n");
        CRC_Completion_Flag = 0xFF;
    }
    else
    {
        printf("CRC Verification Failed\n");
        CRC_Completion_Flag = 0x00;
    }

    while(1);

/* USER CODE END */

    return 0;
}


/* USER CODE BEGIN (4) */
void Data_Initialization(void)
{
    uint32_t i;
    for(i = 0; i < DATA_SIZE; i++)
    {
        Data_Buffer[i] = (i);
    }
    Pre_Determined_CRC[0] = (0x62DEBC3E62FB75A8); /*I calculated this CRC for above data in online using link: https://www.lddgo.net/en/encrypt/crc */
}

uint64_t swap_high32_with_low32( uint64_t val )
{
    uint64_t temp;
    temp = (uint32_t)(val & 0xFFFFFFFF);
    val = ((val >> 32) | (temp << 32));
    return (val);
}
/* USER CODE END */
